/*
  dmp/component.cpp

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "appliance.h"
#include "dmp.h"

namespace ACN::DMP {

/**
 * @brief Appliance::Appliance
 * @param cid
 * @param fctn
 */
Appliance::Appliance(UUID::uuid cid, std::string fctn)
  : SDT::Member(cid, fctn)
  , DMP::Component(cid, fctn)
{
  RlpRegisterVector(DMP_PROTOCOL_ID, std::bind(&Appliance::RlpReceiver, this,
                                               std::placeholders::_1));
}


/**
 * @brief Appliance::RlpReceiver
 * @param root
 */
void Appliance::RlpReceiver(PDU::Message<RLP::Pdu> root)
{
  root->createDataBlock<DMP::Pdu>();
  auto block = std::static_pointer_cast<ACN::PDU::Block<DMP::Pdu>>(root->data());
  DmpReceiver(block);
}


} // namespace ACN::DMP
