/*
  sacn/universemerger.h

  Copyright (c) 2022 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include <universe.h>

namespace sACN {

/**
 * @brief The MergingUniverse class
 */
class UniverseMerger
    : public Universe
{
public:
    explicit UniverseMerger();

    void addSource(std::shared_ptr<Universe>);
    void clear();

    // DMX::Universe overrides
    virtual long age() const override;
    virtual double rxRate() override;
    virtual uint8_t status() const override;
    virtual uint8_t slot(const uint16_t) const override;

    // sACN::Universe overrides
    virtual uint16_t activeSlots() const override;
    virtual std::shared_ptr<DATA::data_header> metadata() const override;
    virtual bool hasSources() const override {return true;} //!< is a poly-source universe @return
    virtual const std::vector<DATA::data_header> sources() const override;


private:
    std::vector<std::shared_ptr<Universe>> sources_;
    std::vector<std::shared_ptr<void>> cb_tokens_;
};

} // namespace sACN
